/*

==============================AI写真==============================
*/
package com.PortraitAi;

import com.alibaba.fastjson.JSON;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLSession;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.MessageDigest;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


public class QueryPortraitTask {

    // 查询AI写真任务接口
    private final String QueryTaskAPIServer = "https://swapface-api.segapi.com";
    private final String QueryTaskAPIUri = "/query";


    private String httpPost(String httpUrl, Map<String, String> headers, String postData) {

        HttpURLConnection connection = null;
        InputStream is = null;
        OutputStream os = null;
        BufferedReader br = null;
        String result = null;
        try {
            URL url = new URL(httpUrl);

            connection = (HttpURLConnection) url.openConnection();
            if (url.getProtocol().toLowerCase().equals("https")) {
                HttpsURLConnection https = (HttpsURLConnection) connection;
                https.setHostnameVerifier(DO_NOT_VERIFY);
                //con = https;
            }

            connection.setRequestMethod("POST");
            connection.setConnectTimeout(15000);
            connection.setReadTimeout(60000);

            connection.setDoOutput(true);

            if (headers != null) {
                for (Map.Entry<String, String> entry : headers.entrySet()) {
                    connection.setRequestProperty(entry.getKey(), entry.getValue());
                }
            }

            os = connection.getOutputStream();
            os.write(postData.getBytes());

            System.out.println("connection.getResponseCode()" + connection.getResponseCode());
            if (connection.getResponseCode() == 200) {
                is = connection.getInputStream();
                br = new BufferedReader(new InputStreamReader(is, "UTF-8"));

                StringBuffer sbf = new StringBuffer();
                String temp = null;
                while ((temp = br.readLine()) != null) {
                    sbf.append(temp);
                    sbf.append("\r\n");
                }
                result = sbf.toString();

            } else {
                System.out.println("bad status code:" + connection.getResponseCode());
                Map<String, List<String>> map = connection.getHeaderFields();
                for (Map.Entry<String, List<String>> entry : map.entrySet()) {
                    System.out.println("Key : " + entry.getKey() +
                            " ,Value : " + entry.getValue());
                }
                is = connection.getErrorStream();
                if (is != null) {
                    br = new BufferedReader(new InputStreamReader(is, "UTF-8"));

                    StringBuffer sbf = new StringBuffer();
                    String temp = null;
                    while ((temp = br.readLine()) != null) {
                        sbf.append(temp);
                        sbf.append("\r\n");
                    }
                    result = sbf.toString();
                }

            }
        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (null != br) {
                try {
                    br.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (null != os) {
                try {
                    os.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (null != is) {
                try {
                    is.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            // 断开与远程地址url的连接
            connection.disconnect();
        }
        return result;
    }


    final static HostnameVerifier DO_NOT_VERIFY = new HostnameVerifier() {

        public boolean verify(String hostname, SSLSession session) {
            return true;
        }
    };

    private byte[] md5Hash(String dataStr) throws Exception {

        dataStr = dataStr;
        MessageDigest md5 = MessageDigest.getInstance("MD5");
        md5.update(dataStr.getBytes("UTF8"));
        byte bytes[] = md5.digest();
        return bytes;
    }

    private String base64(byte[] bytes) {
        return Base64.getEncoder().encodeToString(bytes);
    }


    private String extractValueFromJsonString(String jsonString, String key) {
        String regex = "\"" + key + "\":([^(,|\\}|\\])]*)";
        Matcher matcher = Pattern.compile(regex).matcher(jsonString);
        while (matcher.find()) {
            String val = matcher.group(1);
            val = val.trim();
            if (val.startsWith("\"")) {
                val = val.substring(1);
            }
            if (val.endsWith("\"")) {
                val = val.substring(0, val.length() - 1);
            }
            return val;
        }
        return null;

    }

    private String apiCall(String apiServer, String apiUri, String job_id, String Appcode) {
        try {


            Map bodyJson = new HashMap();
            bodyJson.put("job_id", job_id);

            String body = JSON.toJSONString(bodyJson);

            String bodyMd5 = base64(md5Hash(body));


            String accept = "application/json";
            String contentType = "application/octet-stream; charset=utf-8";

            Map<String, String> headers = new HashMap<String, String>();
            headers.put("Accept", accept);
            headers.put("Authorization", "APPCODE " + Appcode);
            headers.put("Content-Type", contentType);
            headers.put("Content-MD5", bodyMd5);
            UUID uuid = UUID.randomUUID();

            String uuidAsString = uuid.toString();
            headers.put("X-Ca-Nonce", uuidAsString); //UUID.randomUUID()

            String result = httpPost(apiServer + apiUri, headers, body);
            System.out.println("result:" + result);

            if (result == null) {
                System.out.println("httpPost call fail");
                return null;
            }

            System.out.println("result:" + result);
            return result;


        } catch (Exception e) {
            System.out.println("Exception:" + e.getMessage());
            e.printStackTrace();
        }

        return null;
    }

    // 高级写真接口
    public Boolean QueryTask(String Appcode, String job_id) {
        System.out.println("查询写真任务接口 ...");
        try {

            String result = apiCall(this.QueryTaskAPIServer,
                    this.QueryTaskAPIUri,
                    job_id,
                    Appcode);
            if (result == null) {
                return false;
            }


            // 正确做法是使用json解析库，
            // 这里为了减少依赖，直接使用正则表达式解析
            String status = extractValueFromJsonString(result, "status");
            if (!status.equals("0")) {
                System.out.println("bad status:" + status);
                return false;
            }
            return true;
        } catch (Exception e) {
            System.out.println("Exception:" + e.getMessage());
            e.printStackTrace();
        }
        return false;
    }


    public static void main(String[] args) {

        /*
         说明：
             此代码是一份调用接口调用示例，
             PortraitAi也只是一份调用示例的class，
             并不是接口本身，
             请不要把PortraitAi的参数，当成接口本身的参数
        */
        QueryPortraitTask api_test = new QueryPortraitTask();


        String Appcode = "你的appcode";
        String job_id = "34cfe3763b7745aab8223f50a4737615"; //任务ID
        api_test.QueryTask(
                Appcode,
                job_id
        );

        return;
    }

}